//=====================================================================
//
//	j1Conversion.c - common non-standard data conversions
//
//=====================================================================

//=====================================================================
//
// Copyright (c) 2015 Joshua 1 Systems Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
//    1. Redistributions of source code must retain the above copyright notice, this list of
//       conditions and the following disclaimer.
//
//    2. Redistributions in binary form must reproduce the above copyright notice, this list
//       of conditions and the following disclaimer in the documentation and/or other materials
//       provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY JOSHUA 1 SYSTEMS INC. "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
// ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation are those of the
// authors and should not be interpreted as representing official policies, either expressed
// or implied, of Joshua 1 Systems Inc.
//
//=====================================================================

#include "`$INSTANCE_NAME`.h"

#include <CyLib.h>

#include <limits.h>
#include <errno.h>

//---------------------------------------------------------------------
//	j1_u8toa(uint8 Value, char *Buffer)
//---------------------------------------------------------------------

char *j1_u8toa(uint8 Value, char *Buffer)
{
	char	*bfr;
	uint8	i;
	uint8	Digit;
	uint8	Divisor;
	bool	Printed = false;
	
	bfr = Buffer;

	if (Value)
	{
		for(i = 0, Divisor = 100u; i < 3u; i++)
		{
			Digit = Value/Divisor;
			if(Digit || Printed)
			{
				*Buffer++ = '0' + Digit;
				Value -= Digit*Divisor;
				Printed = true;
			}
			Divisor /= 10u;
		}
	}
	else
	{
		*Buffer++ = '0';
	}

	*Buffer = '\0';
	
	return bfr;
}			    

//---------------------------------------------------------------------
//	j1_u16toa(uint16 Value, char *Buffer)
//---------------------------------------------------------------------

char *j1_u16toa(uint16 Value, char *Buffer)
{
	char	*bfr;
	uint8	i;
	uint16	Digit;
	uint16	Divisor;
	bool	Printed = false;
	
	bfr = Buffer;
	
	if (Value)
	{
		for(i = 0, Divisor = 10000u; i < 5u; i++)
		{
			Digit = Value/Divisor;
			if(Digit || Printed)
			{
				*Buffer++ = '0' + Digit;
				Value -= Digit*Divisor;
				Printed = true;
			}
			Divisor /= 10u;
		}
	}
	else
	{
		*Buffer++ = '0';
	}

	*Buffer = '\0';
	
	return bfr;
}			    

//---------------------------------------------------------------------
//	j1_u32toa(uint32 Value, char *Buffer)
//---------------------------------------------------------------------

char *j1_u32toa(uint32 Value, char *Buffer)
{
	char	*bfr;
	uint8	i;
	uint32	Digit;
	uint32	Divisor;
	bool	Printed = false;
	
	bfr = Buffer;

	if (Value)
	{
		for(i = 0, Divisor = 1000000000ul; i < 10; i++)
		{
			Digit = Value/Divisor;
			if(Digit || Printed)
			{
				*Buffer++ = '0' + Digit;
				Value -= Digit*Divisor;
				Printed = true;
			}
			Divisor /= 10ul;
		}
	}
	else
	{
		*Buffer++ = '0';
	}

	*Buffer = '\0';
	
	return bfr;
}

//---------------------------------------------------------------------
//	j1_hexatob(WORD_VAL AsciiChars)
//---------------------------------------------------------------------

uint8 j1_hexatob(j1UINT16_VAL AsciiChars)
{
	// Convert lowercase to uppercase
	if(AsciiChars.B[1] > 'F')
		AsciiChars.B[1] -= 'a'-'A';
	if(AsciiChars.B[0] > 'F')
		AsciiChars.B[0] -= 'a'-'A';

	// Convert 0-9, A-F to 0x0-0xF
	if(AsciiChars.B[1] > '9')
		AsciiChars.B[1] -= 'A' - 10;
	else
		AsciiChars.B[1] -= '0';

	if(AsciiChars.B[0] > '9')
		AsciiChars.B[0] -= 'A' - 10;
	else
		AsciiChars.B[0] -= '0';

	// Concatenate
	return (AsciiChars.B[1]<<4) |  AsciiChars.B[0];
}

//---------------------------------------------------------------------
//	j1_btohexa_high(BYTE b)
//---------------------------------------------------------------------

uint8 j1_btohexa_high(uint8 b)
{
	b >>= 4;
	return (b>0x9u) ? b+'A'-10:b+'0';
}

//---------------------------------------------------------------------
//	j1_btohex_low(BYTE b)
//---------------------------------------------------------------------

uint8 j1_btohexa_low(uint8 b)
{
	b &= 0x0F;
	return (b>9u) ? b+'A'-10:b+'0';
}

//---------------------------------------------------------------------
//	j1_u8tohexstr()
//---------------------------------------------------------------------

char *j1_u8tohexstr(uint8 val, char *buffer)
{
	*buffer++ = j1_btohexa_high(val);
	*buffer++ = j1_btohexa_low(val);
	*buffer   = 0;
	return buffer;
}

//---------------------------------------------------------------------
//	j1_u16tohexstr()
//---------------------------------------------------------------------

char *j1_u16tohexstr(uint16 val, char *buffer)
{
	buffer = j1_u8tohexstr((uint8) (val >> 8)  , buffer);
	return   j1_u8tohexstr((uint8) (val & 0xff), buffer);
}

//---------------------------------------------------------------------
//
//	j1_strtoul() - private version of strtoul - their's has bugs!!
//
//---------------------------------------------------------------------

/*
 * Convert a string to an unsigned long integer.
 *
 * Ignores `locale' stuff.  Assumes that the upper and lower case
 * alphabets and digits are each contiguous.
 */

#define	ISSPACE	isspace
#define	ISDIGIT	isdigit
#define	ISALPHA	isalpha
#define	ISUPPER	isupper

unsigned long j1_strtoul(const char *nptr, char **endptr, int base)
{
	register const char *s = nptr;
	register unsigned long acc;
	register int c;
	register unsigned long cutoff;
	register int neg = 0, any, cutlim;

	/*
	 * See strtol for comments as to the logic used.
	 */
	do {
		c = *s++;
	} while (ISSPACE(c));
	if (c == '-') {
		neg = 1;
		c = *s++;
	} else if (c == '+')
		c = *s++;
	if ((base == 0 || base == 16) &&
	    c == '0' && (*s == 'x' || *s == 'X')) {
		c = s[1];
		s += 2;
		base = 16;
	}
	if (base == 0)
		base = c == '0' ? 8 : 10;
	cutoff = (unsigned long)ULONG_MAX / (unsigned long)base;
	cutlim = (unsigned long)ULONG_MAX % (unsigned long)base;
	for (acc = 0, any = 0;; c = *s++) {
		if (ISDIGIT(c))
			c -= '0';
		else if (ISALPHA(c))
			c -= ISUPPER(c) ? 'A' - 10 : 'a' - 10;
		else
			break;
		if (c >= base)
			break;
		if (any < 0 || acc > cutoff || (acc == cutoff && c > cutlim))
			any = -1;
		else {
			any = 1;
			acc *= base;
			acc += c;
		}
	}
	if (any < 0) {
		acc = ULONG_MAX;
		errno = ERANGE;
	} else if (neg)
		acc = -acc;
	if (endptr != 0)
		*endptr = (char *) (any ? s - 1 : nptr);
	return (acc);
}

//---------------------------------------------------------------------
//	j1_hextonib()
//---------------------------------------------------------------------

bool j1_hextonib(const char hex, uint8 *nib)
{
	if ('0' <= hex && hex <= '9') { *nib = hex - '0';      return true; }
	if ('A' <= hex && hex <= 'F') { *nib = hex - 'A' + 10; return true; }
	if ('a' <= hex && hex <= 'f') { *nib = hex - 'a' + 10; return true; }
	
	return false;
}


//---------------------------------------------------------------------
//	j1_hexfldtouint8()
//---------------------------------------------------------------------

bool j1_hexfldtouint8(const char *fld, uint8 *val)
{
	uint8	hiNib, loNib;

	if (j1_hextonib(fld[0], &hiNib))
	 if (j1_hextonib(fld[1], &loNib))
	{
		*val = hiNib << 4 | loNib;
		return true;
	}
	
	return false;
}

//---------------------------------------------------------------------
//	j1_hexfldtouint16()
//---------------------------------------------------------------------

bool j1_hexfldtouint16(const char *fld, uint16 *val)
{
	j1UINT16_VAL	hex;
	
	if (j1_hexfldtouint8(&fld[0], &hex.B[1]))
	 if (j1_hexfldtouint8(&fld[2], &hex.B[0]))
	{
		*val = hex.Val;
		return true;
	}
	
	return false;
}

//---------------------------------------------------------------------
//	j1_hexfldtouint32()
//---------------------------------------------------------------------

bool j1_hexfldtouint32(const char *fld, uint32 *val)
{
	j1UINT32_VAL	hex;
	
	if (j1_hexfldtouint16(&fld[0], &hex.W[1]))
	 if (j1_hexfldtouint16(&fld[4], &hex.W[0]))
	{
		*val = hex.Val;
		return true;
	}
	
	return false;
}

//---------------------------------------------------------------------
//	j1_mactostr()
//---------------------------------------------------------------------

char *j1_mactostr(uint8 *mac, char *buffer)
{
	char	*bfr;
	char	workBfr[3];
	uint8	i;
	
	bfr = buffer;
	
	for (i = 0; i < 6; i++)
	{
		if (i == 0) *buffer = 0;
		else strcat(buffer, ":");
		
		j1_u8tohexstr(*mac++, workBfr);
		strcat(buffer, workBfr);
	}
	
	return bfr;
}

//---------------------------------------------------------------------
//	j1_iptostr()
//---------------------------------------------------------------------

char *j1_iptostr(uint8 *ip, char *buffer)
{
	char	*bfr;
	char	workBfr[4];
	uint8	i;
	
	bfr = buffer;
	
	for (i = 0; i < 4; i++)
	{
		if (i == 0) *buffer = 0;
		else strcat(buffer, ".");
		
		j1_u8toa(*ip++, workBfr);
		strcat(buffer, workBfr);
	}
	
	return bfr;
}

//---------------------------------------------------------------------
//	j1_strtou8()
//---------------------------------------------------------------------

bool j1_strtou8(const char *str, uint8 *val)
{
	uint32	val32;
	
	if (!j1_strtou32(str, &val32)) return false;
	if (val32 > 255) return false;
	*val = (uint8) val32;
	return true;
}

//---------------------------------------------------------------------
//	j1_strtou16()
//---------------------------------------------------------------------

bool j1_strtou16(const char *str, uint16 *val)
{
	uint32	val32;
	
	if (!j1_strtou32(str, &val32)) return false;
	if (val32 > 65535) return false;
	*val = (uint16) val32;
	return true;
}

//---------------------------------------------------------------------
//	j1_strtou32()
//---------------------------------------------------------------------

bool j1_strtou32(const char *str, uint32 *val)
{
	uint32	val32;
	char	*endPtr;
	
	if (!j1_strtou32ex(str, &val32, &endPtr, 10)) return false;
	if (val32 > 65535) return false;
	*val = (uint16) val32;
	return true;
}

//---------------------------------------------------------------------
//	j1_strtou32ex()
//---------------------------------------------------------------------

#define	ISSPACE	isspace
#define	ISDIGIT	isdigit
#define	ISALPHA	isalpha
#define	ISUPPER	isupper

bool j1_strtou32ex(const char *nptr, uint32 *val, char **endptr, int base)
{
	register const char *s = nptr;
	register uint32 acc;
	register int c;
	register uint32 cutoff;
	register int any, cutlim;

	c = *s++;

	if ((base == 0 || base == 16) && c == '0' && (*s == 'x' || *s == 'X'))
	{
		c = s[1];
		s += 2;
		base = 16;
	}
	
	if (base == 0) { base = c == '0' ? 8 : 10; }
	
	cutoff = (unsigned long)ULONG_MAX / (unsigned long)base;
	cutlim = (unsigned long)ULONG_MAX % (unsigned long)base;
	
	for (acc = 0, any = 0; c; c = *s++)
	{
		     if (ISDIGIT(c)) { c -= '0'; }
		else if (ISALPHA(c)) { c -= ISUPPER(c) ? 'A' - 10 : 'a' - 10; }
		else return false;

		if (c >= base) return false;

		if (acc > cutoff || (acc == cutoff && c > cutlim)) return false;
		
		any  = 1;
		acc *= base;
		acc += c;
	}
	
	if (endptr != 0) { *endptr = (char *) (any ? s - 1 : nptr); }
	
	*val = acc;
	return true;
}
