#!/usr/bin/perl

#----------------------------------------------------------------------
#
#	makej1fsdata.pl - create the .inc or .cyacd file for j1fs system
#
#		perl script to create j1FS .inc file or append hex to
#		a .cyacd file for boot loading into external flash.
#
#		j1FS is a simplified read only disk image structure.
#		it only supports the old 8.3 filename/extension.
#
#		it assumes that .htm files are .ssi
#		to make it easier to associate and edit dynamic
#		web pages as we use for most of our products.
#
#		many things are done brute force and not 'optimized' for
#		speed since this is only a script for processing data
#		at compile time.
#
#		this was based VERY loosely on the original file system
#		that was included with the lwip httpd example
#
#----------------------------------------------------------------------

use strict;
use File::Find;
use Data::Dumper;

# set $baseDir to the base directory path for your project.
# below is an example project location for one our products.
# my $baseDir = "/PSoC/Projects/nECG_DP2_v1_0/nECG_DP2_v1_0.cydsn";
# as soon as we can get the project location from PSoC Creator
# this will be made more automagic
my $baseDir = "/PSoC/Projects/nECG_DP2_v1_0/nECG_DP2_v1_0.cydsn";

# the filesystem is assumed to live in the fsDir off the project
my $fsDir = "fs";

# build the headers into the html files for speed
my $addHeaders = 1;

# set one and only one of the following sets to control output

# output filename & type for include file
# my $outputFile = "j1FS.inc";
# my $outputType = "inc";

# output filename & type for cyacd appending 
my $outputFile = "j1FS.cyacd";
my $outputType = "cyacd";

my $efArrayID  = 0xef;
my $efBaseAddr = 0x00020000;

$Data::Dumper::Terse  = 1;
$Data::Dumper::Indent = 1;

chdir($baseDir);
open(OUTPUT, "> $outputFile");
chdir($fsDir);

my @foundFiles = ();
my %fileTable = ();
my %dirTree = ();

my $cyacdBfr    = '';
my $cyacdBfrCnt = 0;
my $cyacdChkSum = 0;

#----------------------------------------------------------------------
# first find ALL the files (full pathname w/relative start)
#----------------------------------------------------------------------

print "Finding ALL The Files In FS Directory Tree ...\n";

find(\&findEach, ".");

#----------------------------------------------------------------------
#	now find all of the files we want and build %fileTable
#----------------------------------------------------------------------

print "Processing The Found Files We Want ...\n";

foreach my $filePath (@foundFiles)
{
	# Do not include files in CVS directories nor backup files.
	if ($filePath =~ /(CVS|~)/) { next; }
	
	# do not include directories in root starting with '_'
	if ($filePath =~ /^\.\/_/) { next; }
	
	# check for max path size
	if (length($filePath) > 64)
	{
		print "Path too long: $filePath\n";
		<STDIN>;
		exit(0);
	}
	
	# split it into parts
	my @pathParts = split("\/", $filePath);
	
	# check for too few parts
	if (scalar(@pathParts) < 2)
	{
		print "Path too short: $filePath\n";
		<STDIN>;
		exit(0);
	}
	
	# check for first part == '.'
	if ($pathParts[0] ne '.')
	{
		print "Not relative path: $filePath\n";
		<STDIN>;
		exit(0);
	}
	
	for (my $pathIdx = 1; $pathIdx < scalar(@pathParts); $pathIdx++)
	{
		# split into name & extension
		my @nameExt = split(/\./, $pathParts[$pathIdx]);
		
		# add empty extension
		if (scalar(@nameExt) == 1)
		{
			push(@nameExt, "");
		}
		
		# check name/ext size and format
		if (scalar(@nameExt) != 2 || length($nameExt[0]) == 0 || length($nameExt[0]) > 8 || length($nameExt[1]) > 3)
		{
			print "Bad name.ext format: $filePath\n";
			<STDIN>;
			exit(0);
		}
	}
	
	if (exists($fileTable{$filePath}))
	{
		print "Duplicate File: $filePath\n";
		<STDIN>;
		exit(0);
	}
	
	$fileTable{$filePath} = { 'parts' => [@pathParts], 'header' => '', 'fileSize' => 0, 'used' => 0, 'blkCnt' => 0, 'blkStart' => 0 };
	
	# if we want headers built-in each file we need to figure the header text
	if ($addHeaders)
	{
		my $header = '';
		
		if ($filePath =~ /^\.\/400\.htm/)
		{
			$header .= "HTTP/1.0 400 Bad Request\r\n";
		}
		elsif ($filePath =~ /^\.\/401\.htm/)
		{
			$header .= "HTTP/1.0 401 Unauthorized\r\n";
			$header .= "WWW-Authenticate: Basic realm=\"Protected\"\r\n";
		}
		elsif ($filePath =~ /^\.\/404\.htm/)
		{
			$header .= "HTTP/1.0 404 Not Found\r\n";
		}
		elsif ($filePath =~ /^\.\/501\.htm/)
		{
			$header .= "HTTP/1.0 501 Not Implemented\r\n";
		}
		else
		{
			$header .= "HTTP/1.0 200 OK\r\n";
		}
	
		$header .= "Server: j1FS\r\n";
	
		   if ($filePath =~ /\.htm$/) { $header .= "Content-type: text/html\r\nCache-Control: no-cache\r\n\r\n"; }
		elsif ($filePath =~ /\.gif$/) { $header .= "Content-type: image/gif\r\nCache-Control: max-age=3600\r\n\r\n"; }
		elsif ($filePath =~ /\.png$/) { $header .= "Content-type: image/png\r\nCache-Control: max-age=3600\r\n\r\n"; }
		elsif ($filePath =~ /\.jpg$/) { $header .= "Content-type: image/jpeg\r\nCache-Control: max-age=3600\r\n\r\n"; }
		elsif ($filePath =~ /\.css$/) { $header .= "Content-type: text/css\r\nCache-Control: max-age=3600\r\n\r\n"; }
		elsif ($filePath =~ /\.js$/ ) { $header .= "Content-type: application/x-javascript\r\nCache-Control: max-age=3600\r\n\r\n"; }

#		   if ($filePath =~ /\.htm$/) { $header .= "Content-type: text/html\r\nExpires: Fri, 10 Apr 2008 14:00:00 GMT\r\nPragma: no-cache\r\n\r\n"; }
#		elsif ($filePath =~ /\.gif$/) { $header .= "Content-type: image/gif\r\n\r\n"; }
#		elsif ($filePath =~ /\.png$/) { $header .= "Content-type: image/png\r\n\r\n"; }
#		elsif ($filePath =~ /\.jpg$/) { $header .= "Content-type: image/jpeg\r\n\r\n"; }
#		elsif ($filePath =~ /\.css$/) { $header .= "Content-type: text/css\r\n\r\n"; }
#		elsif ($filePath =~ /\.js$/ ) { $header .= "Content-type: application/x-javascript\r\n\r\n"; }
		
		else
		{
			$header = '';
		}
		
		$fileTable{$filePath}{'header'} = $header;
	}
	
	# figure the fileSize, used, blkCnt
	my $fileSize  = -s $filePath;
	$fileSize +=  length($fileTable{$filePath}{'header'});
	my $blkCnt = int(($fileSize + 15) / 16);
	my $used = 16 * $blkCnt;
	
	$fileTable{$filePath}{'fileSize'} = $fileSize;
	$fileTable{$filePath}{'blkCnt'}   = $blkCnt;
	$fileTable{$filePath}{'used'}     = $used;
}

#----------------------------------------------------------------------
#	build the directory tree
#----------------------------------------------------------------------

print "Building Directory Tree ...\n";

foreach my $filePath (sort sortFileTableDepthPath keys(%fileTable))
{
	my %fileEntry = %{$fileTable{$filePath}};
	
	my @parts = @{$fileEntry{'parts'}};
	
	for (my $partIdx = 0; $partIdx < scalar(@parts); $partIdx++)
	{
		my @partsSlice = @parts[0 .. $partIdx];
		
		if ($partIdx < scalar(@parts) - 1)
		{
			my $path = join('/', @partsSlice) . '/';
			
			if (!exists($dirTree{$path}))
			{
				$dirTree{$path} = { 'parts' => [@partsSlice], 'dirs' => {}, 'files' => {}, 'entryCnt' => 0, 'blkCnt' => 0, 'blkStart' => 0 };
			}
			
			if ($partIdx > 0)
			{
				my $parent = join('/', @parts[0 .. $partIdx-1]) . '/';
				
				if (!exists($dirTree{$parent}{'dirs'}{$parts[$partIdx]}))
				{
					$dirTree{$parent}{'dirs'}{$parts[$partIdx]} = $path;
				}
			}
		}
		
		else
		{
			my $parent = join('/', @parts[0 .. $partIdx-1]) . '/';
			
			if (!exists($dirTree{$parent}{'files'}{$parts[$partIdx]}))
			{
				$dirTree{$parent}{'files'}{$parts[$partIdx]} = $parent . $parts[$partIdx];
			}
		}
	}
}

#----------------------------------------------------------------------
#	calculate block info in dirTree
#----------------------------------------------------------------------

print "Calculating Block Info In Directory Tree ...\n";

my $hdrBlkCnt	= 1;
my $dirBlkStart = $hdrBlkCnt;
my $dirBlkCnt	= 0;

foreach my $dirPath (sort sortDirTreeDepthPath keys(%dirTree))
{
	my $entryCnt = scalar(keys(%{$dirTree{$dirPath}{'dirs'}})) + scalar(keys(%{$dirTree{$dirPath}{'files'}}));
	my $blkCnt   = $entryCnt;

	$dirTree{$dirPath}{'entryCnt'} = $entryCnt;
	$dirTree{$dirPath}{'blkCnt'}   = $blkCnt;
	$dirTree{$dirPath}{'blkStart'} = $dirBlkStart;
	
	$dirBlkStart += $blkCnt;
	$dirBlkCnt   += $blkCnt;
}

#----------------------------------------------------------------------
#	update blkStart in fileTable
#----------------------------------------------------------------------

print "Updating Block Start In File Table ...\n";

my $fileBlkStart	= $hdrBlkCnt + $dirBlkCnt;
my $fileBlkCnt		= 0;

foreach my $filePath (sort sortFileTableDepthPath keys(%fileTable))
{
	if ($fileTable{$filePath}{'fileSize'})
	{
		my $blkCnt = $fileTable{$filePath}{'blkCnt'};
		$fileTable{$filePath}{'blkStart'} = $fileBlkStart;
		$fileBlkStart += $blkCnt;
		$fileBlkCnt   += $blkCnt;
	}
}

#----------------------------------------------------------------------
#	if cyacd then initialize its buffer
#----------------------------------------------------------------------

if ($outputType eq 'cyacd')
{
	initCyacdBfr();
}

#----------------------------------------------------------------------
#	first add the header to the output file
#----------------------------------------------------------------------

print "Adding Header To Output ...\n";

my $rootBlk  = $dirTree{'./'}{'blkStart'};
my $rootSize = 16 * $dirTree{'./'}{'entryCnt'};

if ($outputType eq 'cyacd')
{
	my @volname = split("", "j1FS");
	while (scalar(@volname) < 11) { push(@volname, ' '); }
	for (my $h = 0; $h < 11; $h++)
	{
		appendCyacdBfr(unpack("C", $volname[$h]));
	}
	
	appendCyacdBfr(16);
	appendCyacdBfr($rootBlk % 256);
	appendCyacdBfr(int($rootBlk / 256));
	appendCyacdBfr($rootSize % 256);
	appendCyacdBfr(int($rootSize / 256));
}

if ($outputType eq 'inc')
{
	print OUTPUT "static const unsigned char j1FS_blocks[] = {\n\n";
	
	my $hdrText = "\t/*  volume header entry  */\n\t";
	my @volname = split("", "j1FS");
	while (scalar(@volname) < 11) { push(@volname, ' '); }
	for (my $h = 0; $h < 11; $h++)
	{
		if ($h != 0) {$hdrText .= ','; }
		my $byte = $volname[$h];
		$hdrText .= sprintf("0x%02.2x", unpack("C", $byte));
	}
	$hdrText .= sprintf(",0x%02.2x", 16);
	$hdrText .= sprintf(",0x%02.2x", $rootBlk % 256);
	$hdrText .= sprintf(",0x%02.2x", int($rootBlk / 256));
	$hdrText .= sprintf(",0x%02.2x", $rootSize % 256);
	$hdrText .= sprintf(",0x%02.2x", int($rootSize / 256));
	$hdrText .= ",\n\n";
	
	print OUTPUT $hdrText;
}

#----------------------------------------------------------------------
#	next add the directory tree to the output file
#----------------------------------------------------------------------

print "Adding Directory Tree To Output ...\n";

foreach my $dirPath (sort sortDirTreeDepthPath keys(%dirTree))
{
	my %dirBranch  = %{$dirTree{$dirPath}};
	my @dirEntries = ();
	
	foreach my $dirChild (sort(keys(%{$dirBranch{'dirs'}})))
	{
		my $dirChildPath = $dirBranch{'dirs'}{$dirChild};
		
		my %dirChildBranch = %{$dirTree{$dirChildPath}};
		my @dirChildParts = @{$dirChildBranch{'parts'}};
		
		my @nameExt = split(/\./, $dirChildParts[scalar(@dirChildParts) - 1]);
		if (scalar(@nameExt) < 2) { push(@nameExt, ""); }
		my $entryType  = 0x80;
		my $entryStart = $dirChildBranch{'blkStart'};
		my $entrySize  = $dirChildBranch{'entryCnt'} * 16;
	
		push(@dirEntries, { 'nameExt' => [@nameExt], 'type' => $entryType, 'start' => $entryStart, 'size' => $entrySize, 'path' => $dirChildPath });
	}
	
	foreach my $fileChild (sort(keys(%{$dirBranch{'files'}})))
	{
		my $fileChildPath = $dirBranch{'files'}{$fileChild};

		my %fileChildEntry = %{$fileTable{$fileChildPath}};
		my @fileChildParts = @{$fileChildEntry{'parts'}};
		
		my @nameExt = split(/\./, $fileChildParts[scalar(@fileChildParts) - 1]);
		if (scalar(@nameExt) < 2) { push(@nameExt, ""); }
		my $entryType  = 0x00;
		my $entryStart = $fileChildEntry{'blkStart'};
		my $entrySize  = $fileChildEntry{'fileSize'};
	
		push(@dirEntries, { 'nameExt' => [@nameExt], 'type' => $entryType, 'start' => $entryStart, 'size' => $entrySize, 'path' => $fileChildPath });
	}
	
	if ($outputType eq 'cyacd')
	{
		foreach my $dirEntryRef (@dirEntries)
		{
			my %dirEntry      = %{$dirEntryRef};
			
			my @nameExt = @{$dirEntry{'nameExt'}};
			while (length($nameExt[0]) < 8) { $nameExt[0] .= ' '; }
			while (length($nameExt[1]) < 3) { $nameExt[1] .= ' '; }
			my $entryName = $nameExt[0] . $nameExt[1];
			for (my $en = 0; $en < 11; $en++)
			{
				appendCyacdBfr(unpack("C", substr($entryName, $en, 1)));
			}
			
			my $entryType  = $dirEntry{'type'};
			my $entryStart = $dirEntry{'start'};
			my $entrySize  = $dirEntry{'size'};
			
			appendCyacdBfr($entryType);
			appendCyacdBfr($entryStart % 256);
			appendCyacdBfr(int($entryStart / 256));
			appendCyacdBfr($entrySize % 256);
			appendCyacdBfr(int($entrySize / 256));
		}
	}

	if ($outputType eq 'inc')
	{
		# add comment for directory start
		print OUTPUT "\t/*  [ ", substr($dirPath,1), " ]  */\n\n"; 
		
		foreach my $dirEntryRef (@dirEntries)
		{
			# add directory entry
			
			my %dirEntry      = %{$dirEntryRef};
			my $dirEntryText  = "\t/*  " . substr($dirEntry{'path'}, 1) . "  */\n\t";
			
			my @nameExt = @{$dirEntry{'nameExt'}};
			while (length($nameExt[0]) < 8) { $nameExt[0] .= ' '; }
			while (length($nameExt[1]) < 3) { $nameExt[1] .= ' '; }
			my $entryName = $nameExt[0] . $nameExt[1];
			for (my $en = 0; $en < 11; $en++)
			{
				if ($en != 0) {$dirEntryText .= ','; }
				$dirEntryText .= sprintf("0x%02.2x", unpack("C", substr($entryName, $en, 1)));
			}
			
			my $entryType  = $dirEntry{'type'};
			my $entryStart = $dirEntry{'start'};
			my $entrySize  = $dirEntry{'size'};
			
			$dirEntryText .= sprintf(",0x%02.2x", $entryType);
			$dirEntryText .= sprintf(",0x%02.2x", $entryStart % 256);
			$dirEntryText .= sprintf(",0x%02.2x", int($entryStart / 256));
			$dirEntryText .= sprintf(",0x%02.2x", $entrySize % 256);
			$dirEntryText .= sprintf(",0x%02.2x", int($entrySize / 256));
			$dirEntryText .= ",\n\n";
			
			print OUTPUT $dirEntryText;
		}
	}
}

#----------------------------------------------------------------------
#	now add all the data files to the output file
#----------------------------------------------------------------------

print "Adding Data Files To Output ...\n";

foreach my $filePath (sort sortFileTableDepthPath keys(%fileTable))
{
	my %fileEntry = %{$fileTable{$filePath}};

	my $header     = $fileEntry{'header'};
	my $headerLen  = length($header);
	my $fileSize   = $fileEntry{'fileSize'};
	my $dataSize   = $fileSize - $headerLen;

	my $handle = undef;
	open($handle, '<', $filePath);
	binmode($handle);
#	read($handle, my $fileData, $dataSize);
#	close($handle);

#	if ($headerLen) { $fileData = $header . $fileData; }
	
	if ($outputType eq 'cyacd')
	{
		my $pad = 0;
		my $data = 0;

		if ($headerLen)
		{
			for (my $i = 0; $i < $headerLen; $i++)
			{
				appendCyacdBfr(unpack("C", substr($header, $i, 1)));
				$pad++;
			}
		}

		for (my $i = 0; $i < $dataSize; $i++)
		{
			read($handle, $data, 1);
			appendCyacdBfr(unpack("C", $data));
			$pad++;
		}

		# pad to full 16 byte block
		while ($pad % 16 != 0)
		{
			appendCyacdBfr(0x00);
			$pad++;
		}
	}
	
	if ($outputType eq 'inc')
	{
		my $fileText = "\t/*  " . substr($filePath, 1) . "  */";

		# output the file data in hex
		my $pad = 0;
		my $data = 0;
		
		if ($headerLen)
		{
			for (my $i = 0; $i < $headerLen; $i++)
			{
				if ($pad != 0) { $fileText .= ","; }
				if ($pad % 16 == 0) { $fileText .= "\n\t"; }
				$fileText .= sprintf("0x%02.2x", unpack("C", substr($header, $i, 1)));
				$pad++;
			}
		}
	
		for (my $i = 0; $i < $dataSize; $i++)
		{
			if ($pad != 0) { $fileText .= ","; }
			if ($pad % 16 == 0) { $fileText .= "\n\t"; }
			read($handle, $data, 1);
			$fileText .= sprintf("0x%02.2x", unpack("C", $data));
			$pad++;
		}

		# pad to full 16 byte block
		while ($pad % 16 != 0)
		{
			$fileText .= ",";
			if ($pad % 16 == 0) { $fileText .= "\n\t"; }
			$fileText .= sprintf("0x%02.2x", 0);
			$pad++;
		}
	
		my $used		= $fileEntry{'used'};
		my $blkCnt		= $fileEntry{'blkCnt'};
		my $blkStart	= $fileEntry{'blkStart'};
		
		if ($fileSize) { $fileText .= ", // $fileSize / $used / $blkStart / $blkCnt"; }
		$fileText .= "\n\n";
		
		print OUTPUT $fileText;
	}
	
	close($handle);
}

#----------------------------------------------------------------------
#	finish up the output file
#----------------------------------------------------------------------

print "Finishing Up Output File ...\n";

if ($outputType eq 'cyacd')
{
	finishCyacdBfr();
}

if ($outputType eq 'inc')
{
	print OUTPUT "\t/*  Hdr  BlkCnt/Size: ", $hdrBlkCnt, " ", 16 * $hdrBlkCnt, "  */\n";
	print OUTPUT "\t/*  Dir  BlkCnt/Size: ", $dirBlkCnt, " ", 16 * $dirBlkCnt, "  */\n";
	print OUTPUT "\t/*  File BlkCnt/Size: ", $fileBlkCnt, " ", 16 * $fileBlkCnt, "  */\n";
	print OUTPUT "\t/*  Totl BlkCnt/Size: ", $hdrBlkCnt + $dirBlkCnt + $fileBlkCnt, " ", 16 * ($hdrBlkCnt + $dirBlkCnt + $fileBlkCnt), "  */\n";
	print OUTPUT "};\n\n";
}

close(OUTPUT);

#----------------------------------------------------------------------
#	output the stats & wait for return
#----------------------------------------------------------------------

print "Hdr  BlkCnt/Size: ", $hdrBlkCnt, " ", 16 * $hdrBlkCnt, "\n";
print "Dir  BlkCnt/Size: ", $dirBlkCnt, " ", 16 * $dirBlkCnt, "\n";
print "File BlkCnt/Size: ", $fileBlkCnt, " ", 16 * $fileBlkCnt, "\n";
print "Totl BlkCnt/Size: ", $hdrBlkCnt + $dirBlkCnt + $fileBlkCnt, " ", 16 * ($hdrBlkCnt + $dirBlkCnt + $fileBlkCnt), "\n";
<STDIN>;

#----------------------------------------------------------------------
#	findEach()
#----------------------------------------------------------------------

sub findEach
{
	if (-f $_)
	{
		push(@foundFiles, $File::Find::name);
	}
}

#----------------------------------------------------------------------
#	sortDirTreeDepthPath()
#----------------------------------------------------------------------

sub sortDirTreeDepthPath
{
	my $depthA = scalar(@{$dirTree{$a}{'parts'}});
	my $depthB = scalar(@{$dirTree{$b}{'parts'}});
	
	my $diff = $depthA <=> $depthB;
	
	if ($diff != 0) { return $diff; }
	
	return $a cmp $b;
}

#----------------------------------------------------------------------
#	sortFileTableDepthPath()
#----------------------------------------------------------------------

sub sortFileTableDepthPath
{
	my $depthA = scalar(@{$fileTable{$a}{'parts'}});
	my $depthB = scalar(@{$fileTable{$b}{'parts'}});
	
	my $diff = $depthA <=> $depthB;
	
	if ($diff != 0) { return $diff; }
	
	return $a cmp $b;
}

#----------------------------------------------------------------------
#	initCyacdBfr()
#----------------------------------------------------------------------

sub initCyacdBfr
{
	$cyacdBfr    = '';
	$cyacdBfrCnt = 0;
	$cyacdChkSum = 0;
}

#----------------------------------------------------------------------
#	appendCyacdBfr()
#----------------------------------------------------------------------

sub appendCyacdBfr
{
	my ($byte) = @_;
	
	if ($byte > 255)
	{
		print "big byte!!\n\n";
	}
	
	if (length(sprintf("%02.2X", $byte)) != 2)
	{
		print "funny sprintf!!\n\n";
	}

	if ($cyacdBfrCnt == 0x100)
	{
		finishCyacdBfr();
		initCyacdBfr();
	}
	
	$cyacdBfr .= sprintf("%02.2X", $byte);
	$cyacdBfrCnt++;
	$cyacdChkSum += $byte;
	
#	if ($cyacdBfrCnt % 16 == 0) { $cyacdBfr .= ' '; }
}

#----------------------------------------------------------------------
#	finishCyacdBfr()
#----------------------------------------------------------------------

sub finishCyacdBfr
{
	while ($cyacdBfrCnt < 0x0100)
	{
		$cyacdBfr .= sprintf("%02.2X", 0);
		$cyacdBfrCnt++;
	}
	
	if (length($cyacdBfr) != 512)
	{
		print "bad cyacdbfr!!\n\n";
	}
	
	my $prefix = ':';
	
	my $byte = $efArrayID;
	$prefix .= sprintf("%02.2X", $byte);
	$cyacdChkSum += $byte;
	
	my $rowAddr = int($efBaseAddr / 256);
	
	$byte = int($rowAddr / 256);
	$prefix .= sprintf("%02.2X", $byte);
	$cyacdChkSum += $byte;
	
	$byte = $rowAddr % 256;
	$prefix .= sprintf("%02.2X", $byte);
	$cyacdChkSum += $byte;
	
	$byte = int($cyacdBfrCnt / 256);
	$prefix .= sprintf("%02.2X", $byte);
	$cyacdChkSum += $byte;
	
	$byte = $cyacdBfrCnt % 256;
	$prefix .= sprintf("%02.2X", $byte);
	$cyacdChkSum += $byte;
	
	if (length($prefix) != 11)
	{
		print "bad prefix!!\n\n";
	}
	
	$cyacdChkSum &= 0xff;
	$cyacdChkSum  = 256 - $cyacdChkSum;
	$cyacdChkSum &= 0xff;
	my $chkBfr = sprintf("%02.2X", $cyacdChkSum);
	
	if (length($chkBfr) != 2)
	{
		print "bad checksum!!\n\n";
	}
	
	my $bfr = $prefix . $cyacdBfr . $chkBfr;
	
	if (length($bfr) != (11 + 512 + 2))
	{
		print "bad line!!\n\n";
	}
	
	print OUTPUT $bfr, "\n";
	
	$efBaseAddr += $cyacdBfrCnt;
}
